require "opensips-pi/version"
require "open3"
require "shellwords"
require "logger"

module Opensips
  module Pi
    @@logger = nil
    @@bin = nil

    # Returns the OpenSIPS control tool, that is "opensipsctl" by default.
    # You can customize this with {.bin=}.
    #
    # @return [String]
    def self.bin
      @@bin || "opensipsctl"
    end

    # Customize the OpenSIPS control tool. See {.bin} for more information.
    #
    # @example
    #   Opensips::Pi.bin = "/usr/sbin/opensipsctl"
    #   Opensips::Pi.bin #=> "/usr/sbin/opensipsctl"
    # @param cmd [String] Path to OpenSIPS control tool.
    def self.bin=(cmd)
      @@bin = cmd
    end

    # Returns an instance of Ruby's {http://www.ruby-doc.org/stdlib-1.9.3/libdoc/logger/rdoc/Logger.html Logger} (by default)
    # or a logger set with {.logger=}.
    #
    # @return [Logger]
    def self.logger
      @@logger ||= Logger.new(STDOUT)
    end

    # Customize the logger. See {.logger} for more information.
    #
    # @example
    #   Opensips::Pi.logger = Rails.logger
    # @param [Logger]
    def self.logger=(klass)
      @@logger = klass
    end

    # Add a new OpenSIPS user.
    #
    # @param user [String]
    # @param pass [String]
    # @param domain [String]
    def self.add(user, pass, domain=nil)
      call("#{bin} add #{identifier(user,domain).shellescape} #{pass.shellescape}")
    end

    # Remove an user from OpenSIPS.
    #
    # @param user [String]
    # @param domain [String]
    def self.remove(user, domain=nil)
      call("#{bin} rm #{identifier(user,domain).shellescape}")
    end

    # Change user's password.
    #
    # @param user [String]
    # @param pass [String] New password
    # @param domain [String]
    def self.password(user, pass, domain=nil)
      call("#{bin} passwd #{identifier(user,domain).shellescape} #{pass.shellescape}")
    end

    private
    def self.identifier(user, domain)
      domain ? "#{user}@#{domain}" : user
    end

    def self.call(cmd)
      out, err, code = Open3.capture3(cmd)

      logger.info("Opensips::Pi") { out }
      logger.error("Opensips::Pi") { err }

      [code.success?,out,err]
    end
  end
end
